# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,reeb_graph_pairs)
S3method(as_igraph,reeb_graph)
S3method(as_network,reeb_graph)
S3method(as_reeb_graph,igraph)
S3method(as_reeb_graph,network)
S3method(format,reeb_graph)
S3method(format,reeb_graph_pairs)
S3method(print,reeb_graph)
S3method(print,reeb_graph_pairs)
S3method(reeb_graph_pairs,default)
S3method(reeb_graph_pairs,igraph)
S3method(reeb_graph_pairs,network)
S3method(reeb_graph_pairs,reeb_graph)
S3method(reeb_graph_persistence,default)
S3method(reeb_graph_persistence,igraph)
S3method(reeb_graph_persistence,network)
S3method(reeb_graph_persistence,reeb_graph)
S3method(reeb_graph_persistence,reeb_graph_pairs)
export(as_igraph)
export(as_network)
export(as_reeb_graph)
export(read_reeb_graph)
export(reeb_graph)
export(reeb_graph_pairs)
export(reeb_graph_persistence)
import(rJava)
importFrom(phutil,as_persistence)
